<?php

namespace Drupal\config_entity_revisions;

use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\Sql\SqlContentEntityStorage;
use Drupal\Component\Serialization\Json;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\diff\DiffEntityComparison;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\Serializer\Serializer;

/**
 * Controller to make library functions available to various consumers.
 */
abstract class ConfigEntityRevisionsControllerBase extends ControllerBase implements ContainerInjectionInterface, ConfigEntityRevisionsControllerInterface {

  use ConfigEntityRevisionsDataAccessTrait;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Wrapper object for simple configuration from diff.settings.yml.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Wrapper object for simple configuration from diff.settings.yml.
   *
   * @var \Drupal\diff\DiffEntityComparison;
   */
  protected $entityComparison;

  /**
   * Serialiser service.
   *
   * @var Serializer;
   */
  protected $serialiser;

  /**
   * Container instance.
   *
   * @var ContainerInterface
   */
  protected $container;

  /**
   * Constructs a ConfigEntityRevisionsController object.
   *
   * @param ContainerInterface $container
   *   The container interface object.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Config\ImmutableConfig $config
   *   The configuration service.
   * @param \Drupal\diff\DiffEntityComparison $entity_comparison
   *   The diff entity comparison service.
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   *   The entity type manager.
   * @param AccountProxyInterface $current_user
   *   The current user.
   * @param Serializer
   *   The serialiser service.
   */
  public function __construct(
    ContainerInterface $container,
    DateFormatterInterface $date_formatter,
    RendererInterface $renderer,
    ImmutableConfig $config,
    DiffEntityComparison $entity_comparison,
    EntityTypeManager $entity_type_manager,
    AccountProxyInterface $current_user,
    Serializer $serialiser
  ) {
    $this->container = $container;
    $this->dateFormatter = $date_formatter;
    $this->renderer = $renderer;
    $this->config = $this->config('diff.settings');
    $this->entityComparison = $entity_comparison;
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
    $this->serialiser = $serialiser;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container,
      $container->get('date.formatter'),
      $container->get('renderer'),
      $container->get('config.factory')->get('diff.settings'),
      $container->get('diff.entity_comparison'),
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('serializer')
    );
  }

  /**
   * Create an initial revision record.
   *
   * @param ConfigEntityInterface $config_entity
   *   The configuration entity.
   *
   * @return ContentEntityInterface|NULL
   *   The content entity created.
   */
  public function createInitalRevision(ConfigEntityInterface $config_entity) {

    $contentID = $this->getContentEntityIDInConfigEntity($config_entity);

    // Already created.
    if ($contentID) {
      return NULL;
    }

    /**
     * Make a content revisions entity using either the previous version of
     * the config entity or (failing that) the current version.
     * We're doing this here rather than in the update hook because we want
     * to save the reference to the entity config entity version that is being
     * saved now.
     */
    $originalEntity = $this->configEntityStorage()
      ->load($config_entity->id());
    $source = $originalEntity ? $originalEntity : $config_entity;

    $bundle_type = $config_entity->getEntityTypeId() . "_revisions";

    $contentEntity = $this->contentEntityStorage()->create([
      'form' => $source->get('uuid'),
      'configuration' => $this->serialiser->serialize($source, 'json'),
      'revision_uid' => $this->container->get('current_user')->id(),
      'revision_creation_time' => $this->container->get('datetime.time')
        ->getRequestTime(),
      'revision_log_message' => 'Original version when first revision was saved.',
      'moderation_state' => 'draft',
      'type' => $bundle_type,
    ]);

    $contentEntity->save();
    $contentID = $contentEntity->id();

    $this->setContentEntityIDInConfigEntity($config_entity, $contentID);
    $config_entity->save();

    return $contentEntity;
  }

  /**
   * Create revision when a new config entity version is saved.
   *
   * @param ConfigEntityInterface $config_entity
   *   The configuration entity.
   */
  public function createUpdateRevision(ConfigEntityInterface $config_entity) {

    if (!empty($config_entity->revision_id)) {
      $revisionsEntity = $this->contentEntityStorage()
        ->loadRevision($config_entity->revision_id);
    }
    else {
      $contentID = $this->getContentEntityIDInConfigEntity($config_entity);
      $revisionsEntity = $this->contentEntityStorage()->load($contentID);
    }

    $revisionsEntity->set('configuration', $this->serialiser->serialize($config_entity, 'json'));
    $revisionsEntity->setRevisionUserId($this->currentUser->id());
    $revisionsEntity->setRevisionCreationTime($this->container->get('datetime.time')
      ->getRequestTime());

    $new_message = $config_entity->get('revision_log_message')[0]['value'];
    $new_revision = $config_entity->get('revision');
    $moderation_state = $config_entity->get('moderation_state')[0]['value'];
    $published = NULL;

    if (!is_null($moderation_state)) {
      $published = ($moderation_state == 'published');
    }

    if (is_null($moderation_state) && is_null($new_revision)) {
      $new_revision = FALSE;
    }

    if (!is_null($new_message)) {
      $revisionsEntity->setRevisionLogMessage($config_entity->get('revision_log_message')[0]['value']);
    }

    if (!is_null($new_revision)) {
      $revisionsEntity->setNewRevision($new_revision);
    }

    if (!is_null($moderation_state)) {
      $revisionsEntity->moderation_state = $moderation_state;
    }

    if (!is_null($published)) {
      if ($published) {
        $revisionsEntity->setPublished();
      }
      else {
        $revisionsEntity->setUnpublished();
      }

      $revisionsEntity->isDefaultRevision($published);
    }

    $revisionsEntity->save();

  }

  /**
   * Delete revisions when a config entity is deleted.
   *
   * @param ConfigEntityInterface $config_entity
   *   The configuration entity being deleted.
   */
  public function deleteRevisions(ConfigEntityInterface $config_entity) {

    $contentEntity = $this->getContentEntityFromConfigEntity($config_entity);

    if ($contentEntity) {
      $this->contentEntityStorage()->delete([$contentEntity]);
    }
  }

  /**
   * Default, dumb renderer for a revision.
   *
   * @param int $revision_id
   *   The ID of the revision to display.
   *
   * @return array
   *   The resulting render array.
   */
  public function revisionShow(int $revision_id) {
    $revision = $this->contentEntityStorage()->loadRevision($revision_id);

    $build['revision'] = [
      '#theme' => 'markup',
      '#markup' => '<pre>' . print_r(JSON::decode($revision->get('configuration')->value), TRUE) . '</pre>',
    ];
    return $build;
  }

  /**
   * Default implementation providing a title for a rendered revision.
   *
   * @param ConfigEntityInterface $config_entity
   *   The configuration entity being displayed.
   *
   * @return string
   *   The resulting title.
   */
  public function revisionShowTitle(ConfigEntityInterface $config_entity) {
    return '"' . $config_entity->get('title') . '" ' . CONFIG_ENTITY_NAME . ', revision ' . $config_entity->revision_id;
  }

  /**
   * Default implementation providing a title for editing a revision.
   *
   * @param ConfigEntityInterface $config_entity
   *   The configuration entity being displayed.
   *
   * @return string
   *   The resulting title.
   */
  public function revisionEditTitle(ConfigEntityInterface $config_entity, int $revision_id) {
    return 'Edit "' . $config_entity->get('title') . '" ' . CONFIG_ENTITY_NAME . ', revision ' . $revision_id;
  }
}
