<?php

namespace Drupal\config_entity_revisions;

use Drupal\Core\Entity\RevisionableInterface;
use Drupal\Core\ParamConverter\EntityConverter;
use Drupal\Core\ParamConverter\ParamConverterInterface;
use Symfony\Component\Routing\Route;

/**
 * Parameter converter for upcasting entity IDs to full objects.
 *
 * This is useful in cases where the dynamic elements of the path can't be
 * auto-determined; for example, if your path refers to multiple of the same
 * type of entity ("example/{node1}/foo/{node2}") or if the path can act on any
 * entity type ("example/{entity_type}/{entity}/foo").
 *
 * In order to use it you should specify some additional options in your route:
 *
 * @code
 * example.route:
 *   path: foo/{example}
 *   options:
 *     parameters:
 *       example:
 *         type: entity:node
 * @endcode
 *
 * If you want to have the entity type itself dynamic in the url you can
 * specify it like the following:
 * @code
 * example.route:
 *   path: foo/{entity_type}/{example}
 *   options:
 *     parameters:
 *       example:
 *         type: entity:{entity_type}
 * @endcode
 *
 * If your route needs to support pending revisions, you can specify the
 * "load_latest_revision" parameter. This will ensure that the latest revision
 * is returned, even if it is not the default one:
 * @code
 * example.route:
 *   path: foo/{example}
 *   options:
 *     parameters:
 *       example:
 *         type: entity:node
 *         load_latest_revision: TRUE
 * @endcode
 *
 * When dealing with translatable entities, the "load_latest_revision" flag will
 * make this converter load the latest revision affecting the translation
 * matching the content language for the current request. If none can be found
 * it will fall back to the latest revision. For instance, if an entity has an
 * English default revision (revision 1) and an Italian pending revision
 * (revision 2), "/foo/1" will return the former, while "/it/foo/1" will return
 * the latter.
 *
 * @see entities_revisions_translations
 */
class ConfigEntityRevisionsConverterBase extends EntityConverter implements ParamConverterInterface {

  /**
   * {@inheritdoc}
   */
  public function convert($value, $definition, $name, array $defaults) {
    $entity_type_id = $this->getEntityTypeFromDefaults($definition, $name, $defaults);
    $storage = $this->entityManager->getStorage($entity_type_id);
    $entity = $storage->load($value);

    // Get the config_entity_revisions entity, if one exists.
    $revisionsID = $this->getContentEntityIDInConfigEntity($entity);

    if (!$revisionsID) {
      return $entity;
    }

    $storage = $this->entityManager->getStorage('config_entity_revisions');

    // If a specific revision is provided or implied by a submission ID, use it.
    $specific_revision = FALSE;

    if (!empty($defaults['revision_id'])) {
      $specific_revision = $defaults['revision_id'];
    }
    elseif (CONFIG_ENTITY_HAS_OWN_CONTENT && !empty($defaults[CONFIG_ENTITY_CONTENT_PARAMETER_NAME])) {
      // Load the content entity and get the config entity revision from it (if any).
      $content_storage = $this->entityManager->getStorage(CONFIG_ENTITY_CONTENT_ENTITY_TYPE);
      $content = $content_storage->load($defaults[CONFIG_ENTITY_CONTENT_PARAMETER_NAME]);
      if ($content->{CONFIG_ENTITY_CONTENT_PARENT_REFERENCE_FIELD}) {
        $specific_revision = $content->{CONFIG_ENTITY_CONTENT_PARENT_REFERENCE_FIELD}->target_id;
      }
    }

    // If no specific revision has been given, check to see whether we should
    // use the latest revision or the latest published revision.
    if ($specific_revision) {
      $revisionEntity = $storage->loadRevision($specific_revision);
    }
    elseif (array_key_exists('load_latest_revision', $definition)) {
      $revisionEntity = $storage->load($revisionsID);
      $revisionEntity = $this->getLatestRevision($revisionEntity);
    }
    else {
      $revisionEntity = $storage->load($revisionsID);
      $revisionEntity = $this->getLatestPublishedRevision($revisionEntity);

      // If there's no latest published revision and the user has admin
      // permissions, get the latest revision instead.
      if (\Drupal::currentUser()->hasPermission(CONFIG_ENTITY_ADMIN_PERMISSION) &&
        is_null($revisionEntity)) {
        $revisionEntity = $storage->load($revisionsID);
        $revisionEntity = $this->getLatestRevision($revisionEntity);
      }
    }

    if (is_null($revisionEntity)) {
      return NULL;
    }

    $entity = \Drupal::getContainer()->get('serializer')->deserialize(
      $revisionEntity->get('configuration')->value,
      get_class($entity),
      'json');

    // The result of serialising and then deserialising is not an exact
    // copy of the original. This causes problems downstream if we don't fix
    // a few attributes here.
    $entity->set('settingsOriginal', $entity->get('settings'));
    $entity->set('enforceIsNew', FALSE);

    // Record the revision ID in the config entity so we can quickly and
    // easily access the revision record if needed (eg for edit form revision
    // message).
    $entity->set('revision_id', $revisionEntity->getRevisionId());

    return $entity;
  }

  /**
   * Returns the latest published revision of the specified entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface|\Drupal\Core\Entity\RevisionableInterface $entity
   *   The default revision of the entity being converted.
   *
   * @return \Drupal\Core\Entity\EntityInterface|\Drupal\Core\Entity\RevisionableInterface
   *   The latest published revision of the specified entity, or just the
   *   latest revision.
   */
  protected function getLatestPublishedRevision(RevisionableInterface $entity) {
    $revision = NULL;
    $storage = $this->entityManager->getStorage($entity->getEntityTypeId());

    $revision_id = $storage->getLatestPublishedRevisionId($entity->id());
    if ($revision_id) {
      $revision = $this->loadRevision($entity, $revision_id);
    }

    return $revision;
  }

  /**
   * Returns the latest revision of the specified entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface|\Drupal\Core\Entity\RevisionableInterface $entity
   *   The default revision of the entity being converted.
   *
   * @return \Drupal\Core\Entity\EntityInterface|\Drupal\Core\Entity\RevisionableInterface
   *   The latest published revision of the specified entity, or just the
   *   latest revision.
   */
  protected function getLatestRevision(RevisionableInterface $entity) {
    $revision = NULL;
    $storage = $this->entityManager->getStorage($entity->getEntityTypeId());

    $revision_id = $storage->getLatestRevisionId($entity->id());
    if ($revision_id) {
      $revision = $this->loadRevision($entity, $revision_id);
    }

    return $revision;
  }

  /**
   * Loads the specified entity revision.
   *
   * @param \Drupal\Core\Entity\EntityInterface|\Drupal\Core\Entity\RevisionableInterface $entity
   *   The default revision of the entity being converted.
   * @param string $revision_id
   *   The identifier of the revision to be loaded.
   *
   * @return \Drupal\Core\Entity\EntityInterface|\Drupal\Core\Entity\RevisionableInterface
   *   An entity revision object.
   */
  protected function loadRevision(RevisionableInterface $entity, $revision_id) {
    // We explicitly perform a loose equality check, since a revision ID may
    // be returned as an integer or a string.
    if ($entity->getLoadedRevisionId() != $revision_id) {
      $storage = $this->entityManager->getStorage($entity->getEntityTypeId());
      return $storage->loadRevision($revision_id);
    }
    else {
      return $entity;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function applies($definition, $name, Route $route) {
    if (!empty($definition['type']) && strpos($definition['type'], 'entity:') === 0) {
      $entity_type_id = substr($definition['type'], strlen('entity:'));
      if (strpos($definition['type'], '{') !== FALSE) {
        $entity_type_slug = substr($entity_type_id, 1, -1);
        return $name != $entity_type_slug && in_array($entity_type_slug, $route->compile()
            ->getVariables(), TRUE);
      }
      if ($entity_type_id == CONFIG_ENTITY_NAME) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Determines the entity type ID given a route definition and route defaults.
   *
   * @param mixed $definition
   *   The parameter definition provided in the route options.
   * @param string $name
   *   The name of the parameter.
   * @param array $defaults
   *   The route defaults array.
   *
   * @return string
   *   The entity type ID.
   *
   * @throws \Drupal\Core\ParamConverter\ParamNotConvertedException
   *   Thrown when the dynamic entity type is not found in the route defaults.
   */
  protected function getEntityTypeFromDefaults($definition, $name, array $defaults) {
    $entity_type_id = substr($definition['type'], strlen('entity:'));

    // If the entity type is dynamic, it will be pulled from the route defaults.
    if (strpos($entity_type_id, '{') === 0) {
      $entity_type_slug = substr($entity_type_id, 1, -1);
      if (!isset($defaults[$entity_type_slug])) {
        throw new ParamNotConvertedException(sprintf('The "%s" parameter was not converted because the "%s" parameter is missing', $name, $entity_type_slug));
      }
      $entity_type_id = $defaults[$entity_type_slug];
    }
    return $entity_type_id;
  }

  /**
   * Returns a language manager instance.
   *
   * @return \Drupal\Core\Language\LanguageManagerInterface
   *   The language manager.
   *
   * @internal
   */
  protected function languageManager() {
    if (!isset($this->languageManager)) {
      $this->languageManager = \Drupal::languageManager();
      // @todo Turn this into a proper error (E_USER_ERROR) in
      //   https://www.drupal.org/node/2938929.
      @trigger_error('The language manager parameter has been added to EntityConverter since version 8.5.0 and will be made required in version 9.0.0 when requesting the latest translation-affected revision of an entity.', E_USER_DEPRECATED);
    }
    return $this->languageManager;
  }

}
