<?php

namespace Drupal\config_entity_revisions;

use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\config_entity_revisions\Entity\AbstractConfigEntityRevisions;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for reverting a config entity revision.
 */
abstract class ConfigEntityRevisionsDeleteFormBase extends ConfirmFormBase Implements ContainerInjectionInterface {

  /**
   * The config entity.
   *
   * @var \Drupal\Core\Config\Entity\ConfigEntityInterface;
   */
  protected $configEntity;

  /**
   * The ConfigEntityRevisions revision.
   *
   * @var \Drupal\config_entity_revisions\ConfigEntityRevisionsInterface
   */
  protected $revision;

  /**
   * The ConfigEntityRevisions storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $ConfigEntityRevisionsStorage;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Constructs a new ConfigEntityRevisionsRevisionDeleteForm.
   *
   * @param \Drupal\Core\Entity\EntityStorageInterface $ConfigEntityRevisions_storage
   *   The ConfigEntityRevisions storage.
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   */
  public function __construct(EntityStorageInterface $ConfigEntityRevisionsStorage, Connection $connection) {
    $this->ConfigEntityRevisionsStorage = $ConfigEntityRevisionsStorage;
    $this->connection = $connection;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $entity_manager = $container->get('entity.manager');
    return new static(
      $entity_manager->getStorage('config_entity_revisions'),
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return CONFIG_ENTITY_NAME . '_revision_delete_confirm';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return t('Are you sure you want to delete the revision from %revision-date?', ['%revision-date' => format_date($this->revision->getRevisionCreationTime())]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url("entity." . CONFIG_ENTITY_NAME . ".revisions", [ CONFIG_ENTITY_NAME => $this->configEntity->id()]);
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return t('Delete');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, ConfigEntityInterface $config_entity = NULL, $revision_id = NULL) {
    $this->configEntity = $config_entity;
    $this->revision = $this->ConfigEntityRevisionsStorage->loadRevision($revision_id);
    $form = parent::buildForm($form, $form_state);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $was_default = $this->revision->isDefaultRevision();
    $config_entity_revisions_id = $this->getContentEntityIDInConfigEntity($this->configEntity);
    $revisions = $this->connection->select("config_entity_revisions_revision", 'c')
      ->fields('c',['revision'])
      ->condition('id', $config_entity_revisions_id)
      ->execute()
      ->fetchCol();

    // Someone else may have deleted a revision so we can't assume the state
    // when the form was rendered is still valid.
    if (count($revisions) == 1) {
      drupal_set_message(t('There is only one revision remaining. You may not delete it, sorry.'), 'error');
    }
    else {
      if ($was_default) {
        // Change the default to the next newer (if we're deleting the default,
        // there must be no published revisions so it doesn't matter which we
        // choose. Ensure revision_default isn't set on our revision in
        // config_entity_revisions_revision - $was_default can return FALSE
        // even when that value is 1, and that will cause the content moderation
        // module (which does look at that field) to throw an exception.
        $this->connection->update("config_entity_revisions_revision")
          ->condition('id', $config_entity_revisions_id)
          ->fields(['revision_default' => 0])
          ->execute();
        $revision_to_use = ($revisions[0] == $this->revision->getRevisionId()) ?
          $revisions[1] : $revisions[0];
        $new_default = $this->ConfigEntityRevisionsStorage->loadRevision($revision_to_use);
        $new_default->isNew(FALSE);
        $new_default->isDefaultRevision(TRUE);
        $new_default->save();
      }
      $this->ConfigEntityRevisionsStorage->deleteRevision($this->revision->getRevisionId());

      $this->logger(CONFIG_ENTITY_NAME)->notice('Deleted %label revision %revision.', ['%label' => $this->configEntity->label(), '%revision' => $this->revision->getRevisionId()]);
      drupal_set_message(t('Revision from %revision-date of %form has been deleted.', ['%revision-date' => format_date($this->revision->getRevisionCreationTime()), '%form' => $this->configEntity->label()]));
    }

    if (count($revisions) > 2) {
      $form_state->setRedirect(
        'entity.' . CONFIG_ENTITY_NAME . '.revisions',
        [CONFIG_ENTITY_NAME => $this->configEntity->id()]
      );
    }
    else {
      $form_state->setRedirect(
        'entity.' . CONFIG_ENTITY_NAME . '.edit_form',
        [CONFIG_ENTITY_NAME => $this->configEntity->id()]
      );
    }
  }

}
