<?php

namespace Drupal\config_entity_revisions;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\Entity\ConfigEntityInterface;

/**
 * Provides a form for reverting a revision.
 */
class ConfigEntityRevisionsRevertFormBase extends ConfirmFormBase {

  use ConfigEntityRevisionsDataAccessTrait;

  /**
   * The entity type manager.
   *
   * @var EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The content entity revision.
   *
   * @var ConfigEntityRevisionsInterface
   */
  protected $revision;

  /**
   * The config entity.
   *
   * @var ConfigEntityInterface
   */
  protected $config_entity;

  /**
   * The action (publish or revert).
   *
   * @var string
   */
  protected $action;

  /**
   * Constructs a new class instance.
   *
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(EntityTypeManager $entity_type_manager, DateFormatterInterface $date_formatter, TimeInterface $time) {
    $this->entityTypeManager = $entity_type_manager;
    $this->dateFormatter = $date_formatter;
    $this->time = $time;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('date.formatter'),
      $container->get('datetime.time')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return MODULE_NAME . '_revision_revert_confirm';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return t('Are you sure you want to %action to the revision from %revision-date?', [
      '%revision-date' => $this->dateFormatter->format($this->revision->getRevisionCreationTime()),
      '%action' => $this->action,
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('entity.' . CONFIG_ENTITY_NAME . '.revisions', [
      CONFIG_ENTITY_NAME => $this->config_entity->id(),
      'revision_id' => $this->revision->id(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return t(ucfirst($this->action));
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return '';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, ConfigEntityInterface $config_entity = NULL, $revision_id = NULL) {
    $this->config_entity = $config_entity;
    $this->revision = $this->contentEntityStorage()->loadRevision($revision_id);

    $defaultRevision = $this->getContentEntityFromConfigEntity($this->config_entity);
    $any_published = $defaultRevision->isPublished();
    $publish = !$any_published || ($this->revision->getRevisionId() > $defaultRevision->getRevisionId());
    $this->action = $publish ? 'publish' : 'revert';

    $form = parent::buildForm($form, $form_state);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // The revision timestamp will be updated when the revision is saved. Keep
    // the original one for the confirmation message.
    $original_revision_timestamp = $this->revision->getRevisionCreationTime();

    if ($this->action == 'revert') {
      $this->revision = $this->prepareRevertedRevision($this->revision, $form_state);

      $originalLogMessage = $this->revision->getRevisionLogMessage();
      $logMessage = $originalLogMessage ?
        t('Copy of the revision from %date (%message).', [
          '%date' => $this->dateFormatter->format($original_revision_timestamp),
          '%message' => $originalLogMessage,
        ]) :
        t('Copy of the revision from %date.', [
          '%date' => $this->dateFormatter->format($original_revision_timestamp),
        ]);

      $this->revision->setRevisionLogMessage($logMessage);
      $this->revision->setRevisionUserId($this->currentUser()->id());
      $this->revision->setRevisionCreationTime($this->time->getRequestTime());
      $this->revision->setChangedTime($this->time->getRequestTime());
      $this->revision->moderation_state = 'draft';
      $this->revision->setUnpublished();
    }
    else {
      $this->revision->moderation_state = 'published';
      $this->revision->setPublished();
      $this->revision->isDefaultRevision(TRUE);
    }
    $this->revision->save();

    $this->config_entity = \Drupal::getContainer()
      ->get('serializer')
      ->deserialize(
        $this->revision->get('configuration')->value,
        get_class($this->config_entity),
        'json');

    $this->config_entity->enforceIsNew(FALSE);
    $this->config_entity->set('settingsOriginal', $this->config_entity->get('settings'));
    $this->config_entity->set('revision_id', $this->revision->getRevisionId());

    $this->config_entity->save();

    $this->logger('content')
      ->notice('@form: set %title to revision %revision.', [
        '@form' => $this->config_entity->label(),
        '%revision' => $this->revision->getRevisionId(),
      ]);
    drupal_set_message(t('%entity_title %title has been set to the revision from %revision-date.', [
      '%entity_title' => CONFIG_ENTITY_TITLE,
      '%title' => $this->config_entity->label(),
      '%revision-date' => $this->dateFormatter->format($original_revision_timestamp),
    ]));
    $form_state->setRedirect(
      'entity.' . CONFIG_ENTITY_NAME . '.revisions',
      [CONFIG_ENTITY_NAME => $this->config_entity->id()]
    );
  }

  /**
   * Prepares a revision to be reverted.
   *
   * @param ConfigEntityRevisionsInterface $revision
   *   The revision to be reverted.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return ConfigEntityRevisionsInterface
   *   The prepared revision ready to be stored.
   */
  protected function prepareRevertedRevision(ConfigEntityRevisionsInterface $revision, FormStateInterface $form_state) {
    $revision->setNewRevision();
    $revision->isDefaultRevision(TRUE);

    return $revision;
  }

}
