<?php

namespace Drupal\config_entity_revisions\Entity;

use Drupal\Core\Entity\EditorialContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\config_entity_revisions\ConfigEntityRevisionsInterface;

/**
 * Defines the config entity revision entity class.
 *
 * @ContentEntityType(
 *   id = "config_entity_revisions",
 *   label = @Translation("Config Entity Revisions"),
 *   label_collection = @Translation("Config Entity Revisions"),
 *   label_singular = @Translation("config entity revision"),
 *   label_plural = @Translation("Config Entity revisions"),
 *   label_count = @PluralTranslation(
 *     singular = "@count Config Entity revision",
 *     plural = "@count Config Entity revisions"
 *   ),
 *   bundle_label = @Translation("Config Entity Revisions type"),
 *   handlers = {
 *     "moderation" = "Drupal\config_entity_revisions\Entity\Handler\ConfigEntityRevisionsModerationHandler",
 *     "form" = {
 *       "default" = "Drupal\config_entity_revisions\ConfigEntityRevisionForm",
 *       "delete" = "Drupal\config_entity_revisions\Form\ConfigEntityRevisionDeleteForm",
 *       "edit" = "Drupal\config_entity_revisions\ConfigEntityRevisionForm"
 *     },
 *   },
 *   base_table = "config_entity_revisions",
 *   data_table = "config_entity_revisions_field_data",
 *   revision_table = "config_entity_revisions_revision",
 *   revision_data_table = "config_entity_revisions_revision_data",
 *   moderation = "Drupal\config_entity_revisions\Entity\Handler\ConfigEntityRevisionsModerationHandler",
 *   entity_keys = {
 *     "id" = "id",
 *     "revision" = "revision",
 *     "bundle" = "type",
 *     "name" = "name",
 *     "uid" = "uid",
 *     "uuid" = "uuid",
 *     "published" = "published",
 *   },
 *   bundle_entity_type = "config_entity_revisions_type",
 *   permission_granularity = "entity_type",
 *   admin_permission = "administer config_entity_revisions",
 *   links = {
 *     "add-page" = "/config_entity_revisions/add",
 *     "add-form" = "/config_entity_revisions/add/{config_entity_revisions}",
 *     "canonical" = "/config_entity_revisions/{config_entity_revisions}",
 *     "delete-form" = "/config_entity_revisions/{config_entity_revisions}/delete",
 *     "edit-form" = "/config_entity_revisions/{config_entity_revisions}/edit",
 *     "admin-form" = "/admin/structure/config_entity/manage/{config_entity_revisions_bundle}"
 *   }
 * )
 */
class ConfigEntityRevisions extends EditorialContentEntityBase implements ConfigEntityRevisionsInterface {

  /**
   * Value that represents the revision being published.
   */
  const PUBLISHED = 1;

  /**
   * Value that represents the revision being unpublished.
   */
  const NOT_PUBLISHED = 0;

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getPublisher() {
    return $this->get('revision_uid')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getPublisherId() {
    return $this->get('revision_uid')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setPublisherId($uid) {
    $this->set('revision_uid', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getType() {
    return $this->bundle->entity->getType();
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['configuration'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Configuration'))
      ->setDescription(t('The serialised configuration for this revision.'))
      ->setReadOnly(TRUE)
      ->setRequired(TRUE)
      ->setTranslatable(FALSE)
      ->setRevisionable(TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the configuration entity was last edited.'))
      ->setRevisionable(TRUE)
      ->setTranslatable(TRUE);

    return $fields;
  }

  /**
   * Default value callback for 'uid' base field definition.
   *
   * @see ::baseFieldDefinitions()
   *
   * @return array
   *   An array of default values.
   */
  public static function getCurrentUserId() {
    return [\Drupal::currentUser()->id()];
  }

  /**
   * {@inheritdoc}
   */
  public function onPresave(ContentEntityInterface $entity, $default_revision, $published_state) {

    // Updating an existing revision?
    if (is_null($entity->newRevision)) {
      return;
    }

    // Update publishing status if it can be updated and if it needs updating.
    if (($entity instanceof EntityPublishedInterface) && $entity->isPublished() !== $published_state) {
      $published_state ? $entity->setPublished() : $entity->setUnpublished();
    }
  }

}
